@@ -4,10 +4,8 @@ use base 'Encode::Encoding';
 
 use strict;
 
-our $VERSION = do { my @r = (q$Revision: 0.99 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+our $VERSION = do { my @r = (q$Revision: 1.0 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 
-# Just for the time being, we implement jis-7bit
-# encoding via EUC
 
 my $canon = 'iso-2022-kr';
 my $obj = bless {name => $canon}, __PACKAGE__;
@@ -35,34 +33,32 @@ use Encode::CJKConstants qw(:all);
 
 # ISO<->EUC
 
-sub iso_euc {
+sub iso_euc{
     my $r_str = shift;
-    $$r_str =~ s(
-		 ($RE{KSC_5601}|$RE{ISO_ASC})
-		 ([^\e]*)
-		 )
-    {
-	my ($esc, $str) = ($1, $2);
-	if ($esc !~ /$RE{ISO_ASC}/o) {
-	    $str =~ tr/\x21-\x7e/\xa1-\xfe/;
+    $$r_str =~ s/$RE{'2022_KR'}//gox;  # remove the designator 
+    $$r_str =~ s{                    # replace chars. in GL
+     \x0e                            # between SO(\x0e) and SI(\x0f)
+     ([^\x0f]*)                      # with chars. in GR
+     \x0f
 	}
-	$str;
+    {
+			my $out= $1; 
+      $out =~ tr/\x21-\x7e/\xa1-\xfe/;
+      $out;
     }geox;
     $$r_str;
 }
 
 sub euc_iso{
     my $r_str = shift;
-    $$r_str =~ s{
-	($RE{EUC_C}+)
+    substr($$r_str,0,0)=$ESC{'2022_KR'};  # put the designator at the beg. 
+    $$r_str =~ s{                     # move KS X 1001 chars. in GR to GL
+	($RE{EUC_C}+)                       # and enclose them with SO and SI
 	}{
 	    my $str = $1;
-	    my $esc = $ESC{KSC_5601};
 	    $str =~ tr/\xA1-\xFE/\x21-\x7E/;
-	    $esc . $str . $ESC{ASC};
+	    "\x0e" . $str . "\x0f";
 	}geox;
-    $$r_str =~
-	s/\Q$ESC{ASC}\E(\Q$ESC{KSC_5601}\E)/$1/gox;
     $$r_str;
 }
 
